/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic.workflow.api;

import java.util.Date;

import javax.activation.DataHandler;

/**
 * @author daniele
 * 
 */
public interface IQueryAPI {

	//
	// --------> STATIC QUERIES
	// 
	/**
	 * Return the SVG graph of the specified process id. The javascript callback
	 * function specified by "onClickCallbackFuntion" is used for interaction with
	 * each step.
	 * 
	 * @param processId
	 *          Process Id
	 * @param onClickCallbackFuntion
	 *          Javascript callback function. Must be implemented client side.
	 * @return The SVG graph.
	 */
	public String getProcessGraph(long processId);

	/**
	 * Get the process definition identified by "processId"
	 * 
	 * @param processId - The process ID 
	 * @return The process definition
	 */
	public Process getProcess(long processId);


	/**
	 * Gets the latest  process definition for process identified by processName. 
	 * 
	 * @param processName - The process Name
	 * @return The latest definition for process identified by "processName"
	 */
	public Process getProcess(String processName);

	// Get all processes
	public Process[] getProcesses();

	public Process[] getProcesses(boolean onlyStandalone);

	// Get all processes containing a predefined static attribute
	public Process[] getProcessesByFilter(String attributeName, String attributeValue, boolean onlyStandalone);

	public Process[] getLatestDefinitionProcesses();

	/**
	 * Gets the latest definition processes. If onlyStandalone=true, it retrieves
	 * only the processes that can be started as standalone processes (for example
	 * it doesn't retrieve subprocesses).
	 * 
	 * @param onlyStandalone
	 *          If true returns only standalone processes
	 * @return The latest definition processes
	 */
	public Process[] getLatestDefinitionProcesses(boolean onlyStandalone);

	public Process[] getLatestDefinitionProcessesByFilter(String attributeName, String attributeValue,
			boolean onlyStandalone);

	/**
	 * Retrieves all static process properties defined during process design
	 * @param processId Process identifier
	 * @return List of all static process properties. Returns an empty list if no property
	 * is defined
	 */
	public Variable[] getProcessProperties(long processId);

	/**
	 * Retrieves all static process properties defined during process design
	 * @param processName Process name
	 * @return List of all static process properties. Returns an empty list if no property
	 * is defined
	 */
	public Variable[] getProcessProperties(String processName);

	/**
	 * Gets all the human task for the given process
	 * 
	 * @param processId - The process identifier
	 * @return - The array of task definition
	 */
	public Task[] getHumanTask(long processId);

	/**
	 * Gets all the human task for the given process
	 * 
	 * @param processName - The process name
	 * @return - The array of task definition
	 */
	public Task[] getHumanTask(String processName);

	/**
	 * Gets all the human task for the given process
	 * 
	 * @param process - The process object
	 * @return - The array of task definition
	 */
	public Task[] getHumanTask(Process process);

	/**
	 * Gets the human task for the given process, called taskName
	 * 
	 * @param processId - The process identifier
	 * @taskName - The task name
	 * @return - The task definition
	 */
	public Task getHumanTask(long processId, String taskName);

	/**
	 * Gets all the task of the process that are subprocess
	 * 
	 * @param processId - The process identifier
	 * @return - The array of Subprocess Task
	 */
	public SubProcessTask[] getSubprocessTasks(long processId);

	/**
	 * Gets all the task of the process that are subprocess of the given process
	 * 
	 * @param processName - The process name
	 * @return - The array of Subprocess Task
	 */
	public SubProcessTask[] getSubprocessTasks(String processName);

	/**
	 * Gets all the task of the process that are subprocess of the given process
	 * 
	 * @param processName - The process name
	 * @return - The array of Subprocess Task
	 */
	public SubProcessTask[] getSubprocessTasks(Process process);

	/**
	 * Retrieves all processes that are used as subprocesses within the process
	 * identified by processId.
	 * 
	 * @param processId
	 *          Process identifiers
	 * @return The process definitions of the processes used as subprocesses
	 *         within a process.
	 */
	public Process[] getSubProcesses(long processId);

	/**
	 * Retrieves the subprocesses used in a main process, given its name.
	 * 
	 * @param processName
	 *          Process name
	 * @return The process definitions of the processes used as subprocesses
	 *         within a process.
	 */
	public Process[] getSubProcesses(String processName);

	//
	// ---------> DYNAMIC QUERIES ( ProcessInstances and TaskInstances )
	//

	/**
	 * Retrieves all process instances.
	 * 
	 * @return A list of process instances.
	 */
	public ProcessInstance[] getProcessInstances();

	/**
	 * Retrieves all process instances of a specified process.
	 * 
	 * @param processId
	 *          Process Id
	 * @return - The array of Process Instance
	 */
	public ProcessInstance[] getProcessInstances(long processId);

	/**
	 * Retrieves all process instances of a specified process.
	 * 
	 * @param processname
	 *          Process Name
	 * @return - The array of Process Instance
	 */
	public ProcessInstance[] getProcessInstances(String processName);

	/**
	 * Retrieves all process instances that are subprocess instance of the given process instance
	 * 
	 * @param parentProcessInstanceId - The parent process instance id
	 *          
	 * @return A list of process instances.
	 */
	public ProcessInstance[] getSubProcessInstances(long parentProcessInstanceId);

	/**
	 * Get the process instance identified by processInstanceId
	 * 
	 * @param parentProcessInstanceId - The parent process instance id
	 * @return The Process Instance
	 */
	public ProcessInstance getProcessInstanceById(long processInstanceId);

	/**
	 * Retrieves all human task instances (not generated, generated, completed,
	 * cancelled) for a specified process instance.
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @return Array of Task instances
	 */
	public TaskInstance[] getHumanTaskInstances(long processInstanceId);

	/**
	 * Retrieves all human task instances (not generated, generated, completed,
	 * cancelled) for a specified process instance.
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @param actorID
	 *          Identifier of the user that must execute the tasks
	 * @return Array of Task instances
	 */
	public TaskInstance[] getHumanTaskInstancesForActorAndProcess(long processInstanceId, String actorID);

	public TaskInstance[] getHumanTaskInstancesForActor(String actorID);

	/**
	 * Retrieves all human task instances (not generated, generated, completed,
	 * cancelled) for a specified process instance filtering on the task attribute
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @param attributeName - The name of the attribute to use as filter
	 * @param attributeValue - The value to use in filter
	 * @return Array of Task instances
	 */
	public TaskInstance[] getHumanTaskInstances(long processInstanceId, String attributeName, String attributeValue);

	/**
	 * Retrieves the SVG Graph for the given processInstanceId
	 * @param processInstanceId
	 */
	public String getProcessInstanceGraph(long processInstanceId);

	/**
	 * Retrieves all identifiers of process instances linked to the provided one.
	 * 
	 * @param processInstanceId
	 *          Process instance for starting the search
	 * @return Identifiers of linked process instances.
	 */
	public ProcessInstance[] getLinkedProcessInstances(long processInstanceId);

	/**
	 * Retrive the value of the variable related to the "business status" of the
	 * task instance
	 * 
	 * @param processInstanceId
	 * @param taskInstanceId
	 * @return The value of the variable related to the "business status" of the
	 *         task instance
	 */
	public String getTaskInstanceStatus(long processInstanceId, long taskInstanceId);

	/**
	 * Retrive the value of the variable related to the "business status" of a
	 * particular process instance
	 * 
	 * @param processInstanceId
	 * @param taskInstanceId
	 * @return The value of the variable related to the "business status" of the
	 *         task instance
	 */
	public String getProcessInstanceStatus(long processInstanceId);

	/**
	 * Get the values of the condition associatet to outgoing transitions of a task
	 * @param processName - The process Name
	 * @param taskName - The task Name
	 */
	public String[] getHumanTaskOutgoingValues(String processName, String taskName);

	/**
	 * Get the values of the condition associated to outgoing transitions of a task
	 * @param processId - The process Id
	 * @param taskName - The task Name
	 */
	public String[] getHumanTaskOutgoingValues(long processId, String taskName);

	/**
	 * Get name of the generated tasks in the given processInstanceId
	 * @param processInstanceId - The process instance Id
	 */
	public String[] getNextTasksNames(long processInstanceId);

	/**
	 * Get the binary file associated to task if there's one
	 * @param processName - The process name
	 * @param taskName - The task name
	 */
	public DataHandler getTaskDocumentation(String processName, String taskName);

	//
	// --------> TASK LIST
	// 
	/**
	 * Retrieve all the task instances that are in the states of generated or
	 * started.
	 * 
	 * @return TaskInstance[]
	 */
	public TaskInstance[] getGeneratedHumanTaskInstances();

	/**
	 * Retrieve all the task instances filtered by creation date, due date,
	 * process name, task name, state or process instance id. The instances
	 * returned are in the states of generated or started.
	 * 
	 * @param processName
	 * @param taskName
	 * @param createFrom
	 * @param createTo
	 * @param dueDateFrom
	 * @param dueDateTo
	 * @param state
	 * @param processInstanceId
	 * @return TaskInstance[]
	 */
	public TaskInstance[] getGeneratedHumanTaskInstances(String processName, String taskName, Date createFrom,
			Date createTo, Date dueDateFrom, Date dueDateTo, int state, long processInstanceId);

	public TaskInstance[] getHumanTaskInstancesAssignedToActor(String actorId);

	/**
	 * <code>getGeneratedHumanTaskInstances</code> Returns all the
	 * generated human task instances for a given process instance,
	 * whatever the owner of the task is.
	 *
	 * @param processInstanceId a <code>long</code> the id of the
	 * process instance
	 * @return a <code>TaskInstance[]</code> the descriptors of the
	 * tasks of the given process.
	 */
	public TaskInstance[] getGeneratedHumanTaskInstances(long processInstanceId);

}
